#pragma semicolon 1

// Definitions
#define PLUGIN_VERSION "0.6"
#define ACHIEVEMENT_SOUND "misc/achievement_earned.wav"
#define ACHIEVEMENT_PARTICLE "Achieved"

// Misc Definitions
#define DF_CRITS        1048576    //crits = DAMAGE_ACID
#define DF_CRITS_BUFFBANNER        (1<<16)    //16
#define DF_CRITS_JARATE            (1<<22)    //4194304 

// Handles
new Handle:g_Achievements_Server; // Array containing all achievement information from GET. Contains: id, name, description, triggers, blocks
new Handle:g_Blocks_Server; // Array containing all block information from GET. Contains: id, event, target, triggers, requirements
new Handle:g_Achievements_User; // Array containing all achievement information to be sent to POST. Contains: id, steamid, triggers, complete
new Handle:g_Blocks_User; // Array containing all block information to be sent to POST. Contains: id, steamid, triggers, complete
new Handle:g_Achievements_Update; // An array of all achievements that needs to be updated.

// Strings
new String:TempInputString[512];

// Ints
new g_Gamemode = 0;
new Req_Capture_Count = 0;
new TempInputCrit = 0;

// Bools
new Req_Crafted = false;
new bool:g_ClientUpdate[MAXPLAYERS+1] = false;

// Arrays
new g_ClientConditions[MAXPLAYERS+1];
new g_motd[MAXPLAYERS+1] = 0;

// Handles
new Handle:cvar_achinfo;
new Handle:cvar_bot;
new Handle:cvar_debug;
new Handle:cvar_url;
new Handle:cvar_skin;
new Handle:cvar_nosend;

// Includes
#include <sourcemod>
#include <sdktools>
#include <sdkhooks>

#undef REQUIRE_EXTENSIONS
#include <tf2>
#include <tf2_stocks>
#define REQUIRE_EXTENSIONS

#include mw_achievements/misc.inc
#include mw_achievements/sql.inc
#include mw_achievements/webinterface.inc
#include mw_achievements/events.inc
#include mw_achievements/requirement.inc

#include mw_achievements/events/common.inc
#include mw_achievements/events/tf2/common.inc
#include mw_achievements/events/l4d2/common.inc


public Plugin:myinfo = {
    name = "MechaWare Custom Achievements",
    author = "Mecha the Slag",
    description = "Custom Achievements in any Source game!",
    version = PLUGIN_VERSION,
    url = "http://mechaware.net/"
};

public OnPluginStart() {
    g_Achievements_Server = CreateArray();
    g_Achievements_User = CreateArray();
    g_Blocks_Server = CreateArray();
    g_Blocks_User = CreateArray();
    g_Achievements_Update = CreateArray(2);
    
    RegAdminCmd("mc_refresh", Command_Refresh, ADMFLAG_GENERIC, "Refreshes Achievement Info");
    CreateConVar("mc_version", PLUGIN_VERSION, "Version of the plugin");
    cvar_achinfo = CreateConVar("mc_info", "1", "Display achievement description on unlock?");
    cvar_bot = CreateConVar("mc_bot", "0", "Allow bots to earn achievements? (should always be 0)");
    cvar_debug = CreateConVar("mc_debug", "0", "Print debugging");
    cvar_nosend = CreateConVar("mc_nosend", "0", "Do not send achievement updates to server");
    cvar_url = CreateConVar("mc_url", "http://mechaware.net/tf2/achievements/index.php", "Path to the web interface");
    cvar_skin = CreateConVar("mc_skin", "", "Name of a skin to use for the web interface (if none, teamfortress)");
    
    // G A M E  C H E C K //
    decl String:game[32];
    GetGameFolderName(game, sizeof(game));
    g_Gamemode = 0;
    if(StrEqual(game, "tf")) g_Gamemode = 1;
    if(StrEqual(game, "left4dead2")) g_Gamemode = 2;
    
    // Webinterface
    RegConsoleCmd("say", Show_Web);
    
    //LogMessage("Gamemode: %s (%d)", game, g_Gamemode);
    // Hooks
    HookEvent("player_death", Player_Death);
    HookEvent("player_spawn", Player_Spawn);
    RegConsoleCmd("say", Player_Chat);
    RegConsoleCmd("say_team", Player_Chat);
    HookEvent("break_breakable", Break_Breakable);
    
    if (g_Gamemode == 1) { // TF2 Events
        HookEvent("teamplay_point_captured", Point_Capture);
        HookEvent("deploy_buff_banner", Deploy_Buff_Banner);
        HookEvent("player_mvp", Player_Mvp);
        HookEvent("player_sapped_object", Player_Sapped_Object);
        HookEvent("player_healedbymedic", Player_HealedByMedic);
        HookEvent("player_chargedeployed", Player_Chargedeployed);
        HookEvent("item_found", Item_Found);
    }
    if (g_Gamemode == 2) { // L4D2 Events
        HookEvent("pills_used", Pills_Used);
        HookEvent("heal_success", Heal_Success);
        HookEvent("player_entered_checkpoint", Player_Entered_Checkpoint);
        HookEvent("tank_frustrated", Tank_Frustrated);
        HookEvent("player_hurt", Player_Hurt_Native); // This goes in the standard source events common.inc. This is because L4D2 does not support SDKHook's OnTakeDamage.
    }
    
    RefreshDatabaseFull();
}

public OnMapEnd() {
    for (new i = 1; i <= MaxClients; i++) {
        if (IsValidClient(i) && !IsFakeClient(i)) UpdateClient(i);
    }
}

public OnMapStart() {
    PrecacheSound(ACHIEVEMENT_SOUND);
    
    RefreshDatabaseFull();
    ConnectDB_Post();
}

public OnClientPutInServer(client) {
    GetClientDBInfo(client);
    g_motd[client] += 1;
    
    if (g_Gamemode != 2) SDKHook(client, SDKHook_OnTakeDamage, Player_Hurt);
}

public Action:Command_Refresh(client, args) {
    RefreshDatabaseFull();
    return Plugin_Handled;
}

public OnGameFrame() {
    for (new i = 1; i <= MaxClients; i++) {
        UpdateClientCondition(i);
    }
}

public RefreshDatabaseFull() {
    for (new i = 1; i <= MaxClients; i++) {
        if (IsValidClient(i) && !IsFakeClient(i)) UpdateClient(i);
    }
    GetAchievementInfo();
    for (new i = 1; i <= MaxClients; i++) {
        if (IsValidClient(i) && !IsFakeClient(i)) GetClientDBInfo(i);
    }
}

public OnClientDisconnect(client) {
    UpdateClient(client);
}

/////////////////
//N A T I V E S//
/////////////////
// (THRAKA) Thanks Thrawn for levelmod's example on natives and forwards
#if SOURCEMOD_V_MAJOR >= 1 && SOURCEMOD_V_MINOR >= 3
    public APLRes:AskPluginLoad2(Handle:myself, bool:late, String:error[], err_max)
#else
    public bool:AskPluginLoad(Handle:myself, bool:late, String:error[], err_max)
#endif
{
    RegPluginLibrary("mw_ach");

    CreateNative("mw_AchievementEvent", Native_TriggerCustomEvent);

    #if SOURCEMOD_V_MAJOR >= 1 && SOURCEMOD_V_MINOR >= 3
        return APLRes_Success;
    #else
        return true;
    #endif
}

public Native_TriggerCustomEvent(Handle:hPlugin, iNumParams) {
    new String:event[128];
    GetNativeString(1, event, sizeof(event));
    new target0 = GetNativeCell(2);
    new target1 = GetNativeCell(3);
    new target2 = GetNativeCell(4);
    new add = GetNativeCell(5);
    CallAchievementEvent(event,target0, target1, target2, add);
}